local StacKings = _G.StacKings

local L = LibStub("AceLocale-3.0"):GetLocale("StacKings")

local barDefaults = {
	enabled = true,
	name = "New Bar",
	width = 100,
	height = 15,
	unit = "player",
	spell = "",
	auratype = "HELPFUL",
	spec = 0,
	bartexture = "Blizzard",
	fontface = "Friz Quadrata TT",
	fontsize = 12,
	fontcolor = {1, 1, 1},
	fontoutline = nil,
	color = {1,1,1},
	bgcolor = {0.3, 0.3, 0.3},
	bgalpha = 0.8,
	iconenable = true,
	max = 10,
	showTime = true,
	point = { "CENTER", nil, "CENTER", 0, 0 },
	showthreshold = 60,
}

local bars = {
	["SHAMAN"] = {
		[-1] = {
			name = GetSpellInfo(53817), -- Maelstrom Weapon
			spell = GetSpellInfo(53817), -- Maelstrom Weapon
			color = {1,0,1},
			icontexture = select(3, GetSpellInfo(53817)),
			max = 5,
		},
		[-2] = {
			name = GetSpellInfo(49281), -- Lightning Shield
			spell = GetSpellInfo(49281), -- Lightning Shield
			color = {0,0,0.7},
			bgcolor = {1, 0.3, 0.3},
			max = 9,
			point = { "CENTER", nil, "CENTER", 0, -30 },
		},
		[-3] = {
			name = GetSpellInfo(57960), -- Water Shield
			spell = GetSpellInfo(57960), -- Water Shield
			color = {0.2, 0.8, 1},
			bgcolor = {1, 0.3, 0.3},
			max = 4,
			point = { "CENTER", nil, "CENTER", 0, -40 },
		},
	},
	["PRIEST"] = {
		[-1] = {
			name = GetSpellInfo(63737), -- Serendipity
			spell = GetSpellInfo(63737), -- Serendipity
			max = 3,
		},
	},
}

local function table_merge(tab, default)
	for k,v in pairs(default) do
		if not tab[k] then
			if type(v) == 'table' then
				tab[k] = CopyTable(v)
			else
				tab[k] = v
			end
		end
	end
	return tab
end

local _, class = UnitClass("player")
local classBars = bars[class]

if classBars then
	for k,v in pairs(classBars) do
		table_merge(v, barDefaults)
	end
else
	classBars = {}
end

local dbDefaults = {
	profile = {
		bars = classBars,
	},
}

function StacKings:CreateBar()
	local newIndex = #self.db.profile.bars + 1
	local newSettings = CopyTable(barDefaults)
	self.db.profile.bars[newIndex] = newSettings
	self:CreateBarFrame(newIndex)
	self:ApplyFrameSettings(newIndex, newSettings)
	self:UpdateBarOptions()
end

function StacKings:RemoveBar(name)
	self:RemoveBarFrame(name)
	wipe(self.db.profile.bars[name])
	self.db.profile.bars[name] = nil
	self:UpdateBarOptions()
end

function StacKings:ApplyAllSettings()
	for k,v in pairs(self.db.profile.bars) do
		self:ApplyFrameSettings(k, v)
	end
end

function StacKings:OnProfileChanged()
	self:ApplyAllSettings()
	self:UpdateBarOptions()
end

function StacKings:InitializeDB()
	local db = LibStub("AceDB-3.0"):New("StacKingsDB", dbDefaults)
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db = db
end